@@ -28,6 +28,7 @@
 from copy import deepcopy
 import logging
 import pickle
+import warnings
 
 import ldap
 try:
@@ -216,6 +217,8 @@ class LDAPTest(TestCase):
         cls.configure_logger()
         cls.mockldap = mockldap.MockLdap(cls.directory)
 
+        warnings.filterwarnings('ignore', message='.*?AUTH_PROFILE_MODULE', category=DeprecationWarning, module='django_auth_ldap')
+
     @classmethod
     def tearDownClass(cls):
         del cls.mockldap
@@ -231,6 +234,10 @@ class LDAPTest(TestCase):
         self.mockldap.stop()
         del self.ldapobj
 
+    #
+    # Tests
+    #
+
     def test_options(self):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -271,6 +278,25 @@ class LDAPTest(TestCase):
             ['initialize', 'simple_bind_s']
         )
 
+    def test_default_settings(self):
+        class MyBackend(backend.LDAPBackend):
+            default_settings = dict(
+                USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test'
+            )
+        self.backend = MyBackend()
+
+        user_count = User.objects.count()
+
+        user = self.backend.authenticate(username='alice', password='password')
+
+        self.assertTrue(not user.has_usable_password())
+        self.assertEqual(user.username, 'alice')
+        self.assertEqual(User.objects.count(), user_count + 1)
+        self.assertEqual(
+            self.ldapobj.methods_called(),
+            ['initialize', 'simple_bind_s']
+        )
+
     def test_simple_bind_escaped(self):
         """ Bind with a username that requires escaping. """
         self._init_settings(
@@ -611,12 +637,14 @@ class LDAPTest(TestCase):
         def handle_populate_user(sender, **kwargs):
             self.assertTrue('user' in kwargs and 'ldap_user' in kwargs)
             kwargs['user'].populate_user_handled = True
-        backend.populate_user.connect(handle_populate_user)
 
+        backend.populate_user.connect(handle_populate_user)
         user = self.backend.authenticate(username='alice', password='password')
 
         self.assertTrue(user.populate_user_handled)
 
+        backend.populate_user.disconnect(handle_populate_user)
+
     @unittest.skipIf(django.VERSION >= (1, 7), "Skip profile tests in Django>=1.7")
     def test_signal_populate_user_profile(self):
         settings.AUTH_PROFILE_MODULE = 'django_auth_ldap.TestProfile'
@@ -635,11 +663,31 @@ class LDAPTest(TestCase):
 
         django.db.models.signals.post_save.connect(handle_user_saved, sender=User)
         backend.populate_user_profile.connect(handle_populate_user_profile)
-
         user = self.backend.authenticate(username='alice', password='password')
 
         self.assertTrue(user.get_profile().populated)
 
+        backend.populate_user_profile.disconnect(handle_populate_user_profile)
+        django.db.models.signals.post_save.disconnect(handle_user_saved, sender=User)
+
+    def test_signal_ldap_error(self):
+        self._init_settings(
+            BIND_DN='uid=bob,ou=people,o=test',
+            BIND_PASSWORD='bogus',
+            USER_SEARCH=LDAPSearch(
+                "ou=people,o=test", ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
+            )
+        )
+
+        def handle_ldap_error(sender, **kwargs):
+            self.assertEqual(kwargs['context'], 'authenticate')
+            raise kwargs['exception']
+
+        backend.ldap_error.connect(handle_ldap_error)
+        with self.assertRaises(ldap.LDAPError):
+            self.backend.authenticate(username='alice', password='password')
+        backend.ldap_error.disconnect(handle_ldap_error)
+
     def test_no_update_existing(self):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -1056,6 +1104,22 @@ class LDAPTest(TestCase):
 
         self.assertEqual(self.backend.get_group_permissions(alice), set(["auth.add_user", "auth.change_user"]))
 
+    def test_authorize_external_unknown(self):
+        self._init_settings(
+            USER_SEARCH=LDAPSearch(
+                "ou=people,o=test", ldap.SCOPE_SUBTREE, '(uid=%(user)s)'
+            ),
+            GROUP_SEARCH=LDAPSearch('ou=groups,o=test', ldap.SCOPE_SUBTREE),
+            GROUP_TYPE=MemberDNGroupType(member_attr='member'),
+            FIND_GROUP_PERMS=True,
+            AUTHORIZE_ALL_USERS=True
+        )
+        self._init_groups()
+
+        alice = User.objects.create(username='not-in-ldap')
+
+        self.assertEqual(self.backend.get_group_permissions(alice), set())
+
     def test_create_without_auth(self):
         self._init_settings(
             USER_DN_TEMPLATE='uid=%(user)s,ou=people,o=test',
@@ -1212,6 +1276,10 @@ class LDAPTest(TestCase):
         self.assertTrue(self.backend.has_perm(alice, "auth.add_user"))
         self.assertTrue(self.backend.has_module_perms(alice, "auth"))
 
+    #
+    # Utilities
+    #
+
     def _init_settings(self, **kwargs):
         self.backend.settings = TestSettings(**kwargs)
 
